import json
from django.http import HttpResponse
from rest_framework import viewsets

from djangoauthtoken.models import TokenUser
from djangoauthtoken.serializers import UserSerializer

class UserViewSet(viewsets.ModelViewSet):
    queryset = TokenUser.objects.all()
    serializer_class = UserSerializer



import jwt

from django.conf import settings
from django.db import models
from django.conf import settings

from djangoauthtoken.models import Base, TokenUser
from djangoauthtoken.utils import get_epoch

class Token(Base):

    token = models.CharField(max_length=Base.MAX_LENGTH_SMALL)
    refresh_token = models.CharField(max_length=Base.MAX_LENGTH_SMALL)
    expiry_time = models.IntegerField(default=settings.EXPIRY)
    user = models.ForeignKey(TokenUser, on_delete=models.CASCADE)

    def save(self, *args, **kwargs):
        epoch_time = get_epoch()
        self.token = jwt.encode({
                "user_id": self.user.id,
                "username": self.user.username,
                "type": settings.AUTH_TOKEN,
                "date": epoch_time
                },
                settings.JWT_SECRET,
                algorithm=settings.JWT_ALGO
                )

        self.refresh_token = jwt.encode({
            "user_id": self.user.id,
            "username": self.user.username,
            "type": settings.REFRESH_TOKEN,
            "date": epoch_time
            },
            settings.JWT_SECRET,
            algorithm=settings.JWT_ALGO
            )
        super(Token, self).save(*args, **kwargs)


from django.db import models

class CaseInsensitiveEmailField(models.EmailField):
    """
    """

    def to_python(self, value):
        """
        """

        value  = super(CaseInsensitiveEmailField, self).to_python(value)
        return value.lower() if isinstance(value, str) else value


from djangoauthtoken.views.user import UserViewSet
from djangoauthtoken.views.login import login
from djangoauthtoken.views.refresh_token import refresh_token
from djangoauthtoken.views.sign_up import sign_up



from django.core.exceptions import ObjectDoesNotExist
from django.contrib import auth
from django.views.decorators.csrf import csrf_exempt
from rest_framework import status
from rest_framework.decorators import api_view, authentication_classes, permission_classes
from rest_framework.permissions import AllowAny
from rest_framework.response import Response

from djangoauthtoken.models import TokenUser, Token
from djangoauthtoken.utils import get_or_create_csrf_token

@api_view(["POST"])
@authentication_classes([])
@permission_classes([])
@csrf_exempt
def login(request):
    """
    Login view
    It expects username and password.
    #TODO: Flag to switch to Email.
    """
    data = request.data
    username = data['username']
    password = data['password']
    try:
        if auth.authenticate(username=username, password=password):
            
            user = TokenUser.objects.get(username=username)
            user_token = Token(user=user)
            user_token.save()
            _csrf = get_or_create_csrf_token(request)

            return Response({
                "id": user.id,
                "username": user.username,
                "email": user.email,
                "token": user_token.token,
                "refresh_token": user_token.refresh_token,
                "expires_at": user_token.expiry_time
                },
                status=status.HTTP_200_OK, 
                headers={
                    'X-CSRFToken': _csrf
                })
        else:
            # Create your own Exception layer.
            raise ObjectDoesNotExist
    except ObjectDoesNotExist:
        return Response(status=status.HTTP_404_NOT_FOUND)




from rest_framework import serializers
from djangoauthtoken.models import TokenUser as User

# Serializers define the API representation.
class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        exclude = [
            'password', 
            'last_login',
            'is_superuser',
            'is_staff',
            'is_active',
            'groups',
            'user_permissions'
            ]


from datetime import datetime

from rest_framework import exceptions
from rest_framework.authentication import BaseAuthentication, get_authorization_header

from djangoauthtoken.models import Token

class CustomTokenAuthentication(BaseAuthentication):
    """
    Simple token based authentication.

    Clients should authenticate by passing the token key in the "Authorization"
    HTTP header, prepended with the string "Token ".  For example:

        Authorization: Bearer 401f7ac837da42b97f613d789819ff93537bee6a
    """
    keyword = 'Bearer'

    def authenticate(self, request):
        
        auth = get_authorization_header(request).split()
        if not auth or auth[0].lower() != self.keyword.lower().encode():
            return None

        if len(auth) == 1:
            msg = 'Invalid token header. No credentials provided.'
            raise exceptions.AuthenticationFailed(msg)
        elif len(auth) > 2:
            msg = 'Invalid token header. Token string should not contain spaces.'
            raise exceptions.AuthenticationFailed(msg)

        try:
            token = auth[1].decode()
        except UnicodeError:
            msg = 'Invalid token header. Token string should not contain invalid characters.'
            raise exceptions.AuthenticationFailed(msg)

        return self.authenticate_credentials(token)
    
    def authenticate_credentials(self, key):
        try:
            # Check for token.
            token = Token.objects.select_related('user').get(token=key)

            # check for expiry time.
            now = datetime.utcnow()
            _diff = now - token.created_at.replace(tzinfo=None)
            if _diff.total_seconds() > token.expiry_time:
                msg = 'Token expired.'
                raise exceptions.AuthenticationFailed(msg)
        except Token.DoesNotExist:
            raise exceptions.AuthenticationFailed('Invalid token.')

        return (token.user, token)
    
    def authenticate_header(self, request):
        return self.keyword



from django.middleware import csrf

def get_or_create_csrf_token(request) -> str:
    token = request.META.get('CSRF_COOKIE', None)
    if token is None:
        token = csrf._get_new_csrf_string()
    return token


from django.contrib.auth.models import AbstractUser
import pytz
import datetime

from django.db import models
from django.core.validators import RegexValidator

from djangoauthtoken.models import Base
from djangoauthtoken.models.custom_fields import CaseInsensitiveEmailField

tz = [(item, datetime.datetime.now(pytz.timezone(item)).strftime("%z") + " " + item ) for item in pytz.all_timezones ]

phone_regex = RegexValidator(
    regex=r'^\+?1?\d{9,15}',
    message="Phone numbers must be in International format: '+14169058762'. Up to 15 digits "
)

class TokenUser(AbstractUser, Base):
    email = CaseInsensitiveEmailField(max_length=Base.MAX_LENGTH_MEDIUM)
    phone_number = models.CharField(validators=[phone_regex], max_length=17)
    
    def __str__(self) -> str:
        return self.email
    




from rest_framework import status
from rest_framework.decorators import api_view, authentication_classes, permission_classes
from rest_framework.response import Response

from djangoauthtoken.models import TokenUser

@api_view(["POST"])
@authentication_classes([])
@permission_classes([])
def sign_up(request):
    """
    Sign up request
    """
    try:
        data = request.data
        username = data['username']
        email = data['email']
        password = data['password']

        user, exists = TokenUser.objects.get_or_create(
            username=username,
            email=email
        )
        user.set_password(password)
        user.save()

        if not exists:
            raise Exception
        
        return Response({
            "username": user.username,
            "email": user.email,
            "id": user.id
        }, status=status.HTTP_201_CREATED)
    except:
        return Response(status=status.HTTP_400_BAD_REQUEST)




import jwt

from rest_framework import status
from rest_framework.decorators import api_view, authentication_classes, permission_classes
from rest_framework.response import Response
from django.conf import settings

from djangoauthtoken.models import TokenUser, Token

@api_view(["POST"])
@authentication_classes([])
@permission_classes([])
def refresh_token(request):
    """
    Refresh Token request.
    """
    try:
        data = request.data
        refresh_token = data['refresh_token']
        refresh_token_decode = jwt.decode(refresh_token,
                                        settings.JWT_SECRET,
                                        algorithms=settings.JWT_ALGO)

        print(refresh_token_decode)

        user = TokenUser.objects.get(id=refresh_token_decode['user_id'])
        user_token = Token.objects.get(user=user, refresh_token=refresh_token)
        token = Token(user=user)
        token.save()
        print(token.id)

        return Response({
                    "id": user.id,
                    "username": user.username,
                    "email": user.email,
                    "token": token.token,
                    "refresh_token": token.refresh_token,
                    "expires_at": token.expiry_time
                    },
                    status=status.HTTP_200_OK)
    except:
        return Response(status=status.HTTP_400_BAD_REQUEST)




#!/usr/bin/env python

import sys
import os
from os.path import dirname
from setuptools import find_packages, setup

BASE_DIR = os.path.dirname(os.path.realpath(__file__))

if os.path.exists(BASE_DIR) and os.path.isdir(BASE_DIR):
    sys.path.insert(0, BASE_DIR)
else:
    raise ValueError('Error in Path')


def get_file_contents(filename):
    with open(os.path.join(dirname(__file__), filename)) as fp:
        return fp.read()


def get_install_requires():
    requirements = get_file_contents('requirements.txt')
    install_requires = []
    for line in requirements.split('\n'):
        line = line.strip()
        if line and not line.startswith('-'):
            install_requires.append(line)
    return install_requires


setup(
    name='auth-token-django',
    description='Django token authentication',
    version = "0.0.3",
    author='Shivin Agarwal',
    long_description=get_file_contents('README.md'),
    author_email='shivin.agarwal15@gmail.com',
    url='https://github.com/Shivin01/django_auth_token',
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 3.9'
    ],
    install_requires=get_install_requires(),
    include_package_data=True,
    keywords='django auth token',
    packages=find_packages(exclude=['tests*']),
    package_data={
        # If any package contains *.so or *.pyi or *.lic files or *.key files,
        # include them:
        "": ["*.so", "*.pyi", "*.lic", "*.key", "*.html"],
    },
)